import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:flutter/material.dart';
import 'package:jwt_decoder/jwt_decoder.dart';
import 'package:shared_preferences/shared_preferences.dart';

class AuthProvider with ChangeNotifier {
  Future<void> logout() async {
    final preferences = await SharedPreferences.getInstance();
    preferences.remove(TOKEN_KEY);
    notifyListeners();
  }

  Future<String?> getToken() async {
    final preferences = await SharedPreferences.getInstance();
    String? token = preferences.getString(TOKEN_KEY);
    return token;
  }

  Future<Map<String, dynamic>> getTokenData() async {
    final token = await getToken();
    if (token == null) return {};
    final data = JwtDecoder.decode(token);
    if (data["Role"] != null) {
      data["role"] = UserType.from[data["Role"]];
    }

    return data;
  }

  Future<bool> isAuthenticated() async {
    return await getToken() != null;
  }

  Future<void> setToken(String token) async {
    final preferences = await SharedPreferences.getInstance();
    preferences.setString(TOKEN_KEY, token);
    notifyListeners();
  }
}
