import 'dart:async';

import 'package:apus_drones_mobile/constants/OrderStatus.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/PartnerOrderModel.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/services/LocalNotification.dart';
import 'package:apus_drones_mobile/services/PartnerService.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:apus_drones_mobile/widgets/PartnerOrderCard.dart';
import 'package:apus_drones_mobile/widgets/PartnerOrderModal.dart';
import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:modal_bottom_sheet/modal_bottom_sheet.dart';

class PartnerOrdersScreen extends StatefulWidget {
  static final String routeName = 'partner/orders';
  static final int botNavItemIndex = 3;

  @override
  _PartnerOrdersScreenState createState() => _PartnerOrdersScreenState();
}

class _PartnerOrdersScreenState extends State<PartnerOrdersScreen> {
  List<PartnerOrderModel> orderList = [];
  PartnerService service = new PartnerService();
  bool show = false;
  bool modalOpen = false;
  late PartnerOrderModel order;

  @override
  void initState() {
    super.initState();
    orderList = [];
    service = new PartnerService();
    show = false;
    modalOpen = false;
    this._fetchOrders('');
    startTimer();
  }

  void startTimer() {
    tick(callNewOrders, 30, 1);
    tick(showWaitingOrdersModal, 15, 2);
  }

  void tick(Function func, int startTime, int tickId) {
    int time = startTime;
    const oneSec = const Duration(seconds: 1);
    Timer.periodic(oneSec, (Timer timer) {
      if (!mounted) {
        timer.cancel();
        return;
      }

      if (ModalRoute.of(context)!.isCurrent) {
        this.setState(() {});
        if (time <= 0) {
          time = startTime;
          func();
        }
        time--;
        print('Tick ID $tickId tempo: $time');
      }
    });
  }

  void callNewOrders() {
    this._fetchOrders('WAITING_FOR_PARTNER');
    if (show) {
      LocalNotification n = LocalNotification();
      n.displayNotification(order.customer.name);
    }
  }

  void showWaitingOrdersModal() {
    if (!modalOpen) {
      for (int i = 0; i < orderList.length; i++) {
        if (orderList[i].status == OrderStatus.WAITING_FOR_PARTNER) {
          order = orderList[i];
          show = true;
          i = orderList.length;
        }
      }
    }
  }

  _fetchOrders(String filter) {
    service.getAllOrders(filter).then((orders) => this.setState(() {
          for (int i = 0; i < orders.length; i++) {
            orderList.insert(0, orders[i]);
          }
        }));
  }

  void changeModalOpenToFalse() {
    modalOpen = false;
  }

  void showModal(BuildContext context) {
    if (show) {
      show = false;
      modalOpen = true;
      SchedulerBinding.instance!.addPostFrameCallback((_) {
        showMaterialModalBottomSheet(
            context: context,
            builder: (context) =>
                PartnerOrderModal(order, refresh, changeModalOpenToFalse));
      });
    }
  }

  void refresh() {
    this.setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    showModal(context);
    return Scaffold(
      appBar: Appbar(
        title: "Meus Pedidos",
      ),
      bottomNavigationBar: BotNavBar(role: UserType.PARTNER),
      body: Center(
        child: Column(
          children: [
            Container(
                constraints: BoxConstraints(maxWidth: 200),
                child: Padding(
                  padding: EdgeInsets.all(16),
                  child: Text(
                    "Os pedidos estão ordenados por data, você verá os pedidos mais recentes primeiro",
                    textAlign: TextAlign.center,
                    style: TextStyle(fontSize: 12, fontStyle: FontStyle.italic),
                  ),
                )),
            Expanded(
              child: ListView.builder(
                itemBuilder: (ctx, index) => PartnerOrderCard(
                  model: this.orderList[index],
                ),
                itemCount: this.orderList.length,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
