import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/ProductModel.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/screens/EditProductScreen.dart';
import 'package:apus_drones_mobile/services/PartnerService.dart';
import 'package:apus_drones_mobile/services/ProductService.dart';
import 'package:apus_drones_mobile/utils/money.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:apus_drones_mobile/widgets/PartnerProductCard.dart';
import 'package:flutter/material.dart';

class PartnerProductListScreen extends StatefulWidget {
  static final routeName = 'partner/products';
  const PartnerProductListScreen({Key? key}) : super(key: key);

  @override
  _PartnerProductListScreenState createState() =>
      _PartnerProductListScreenState();
}

class _PartnerProductListScreenState extends State<PartnerProductListScreen> {
  List<ProductModel> products = [];
  List<bool> selectedProductsIndexes = [];
  PartnerService partnerService = new PartnerService();
  ProductService productService = new ProductService();

  _fetchProducts() {
    partnerService.getProducts().then((data) => this.setState(() {
          products = data;
          selectedProductsIndexes = List.filled(data.length, false);
        }));
  }

  List<ProductModel> _getSelectedProducts() {
    return selectedProductsIndexes
        .asMap()
        .entries
        .where((element) => element.value)
        .map((element) => products[element.key])
        .toList();
  }

  _deleteProducts() async {
    await Future.wait(
        _getSelectedProducts().map((p) => productService.deleteProduct(p)));

    _fetchProducts();
  }

  bool _isOnMultiSelectionMode() {
    return selectedProductsIndexes.any((s) => s);
  }

  _toggleProductSelection(index) {
    setState(() {
      selectedProductsIndexes[index] = !selectedProductsIndexes[index];
    });

    print(
        'Produtos selecionados: ${selectedProductsIndexes.asMap().entries.where((e) => e.value).map((e) => e.key).toList()}');
  }

  _getFab() => _isOnMultiSelectionMode()
      ? FloatingActionButton.extended(
          onPressed: () => popUpDeleteScreen(context),
          label: Icon(Icons.delete),
          shape: CircleBorder(),
        )
      : FloatingActionButton.extended(
          onPressed: () {
            Navigator.push(
              context,
              MaterialPageRoute(builder: (context) => EditProductScreen()),
            );
          },
          label: Icon(Icons.add),
          shape: CircleBorder(),
        );

  @override
  void initState() {
    super.initState();
    this._fetchProducts();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: Appbar(
        title: "Meus Produtos",
      ),
      body: Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
        child: Column(
          children: [
            Expanded(
              child: ListView.builder(
                padding: EdgeInsets.only(bottom: 30),
                itemCount: this.products.length,
                itemBuilder: (context, index) => PartnerProductCard(
                    product: this.products[index],
                    selected: this.selectedProductsIndexes[index],
                    onTap: () {
                      _isOnMultiSelectionMode()
                          ? _toggleProductSelection(index)
                          : Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => EditProductScreen(
                                  id: this.products[index].id,
                                ),
                              ),
                            );
                    },
                    onLongPress: () {
                      _toggleProductSelection(index);
                      // popUpDeleteScreen(context, [products[index]]);
                    }),
              ),
            ),
          ],
        ),
      ),
      bottomNavigationBar: BotNavBar(role: UserType.PARTNER),
      floatingActionButton: _getFab(),
      floatingActionButtonLocation: FloatingActionButtonLocation.endFloat,
    );
  }

  popUpDeleteScreen(BuildContext context) {
    final selectedProducts = _getSelectedProducts();

    Widget cancelButton = ElevatedButton(
      style: ElevatedButton.styleFrom(
          primary: Colors.grey.shade100,
          onPrimary: Colors.black,
          textStyle: TextStyle(
            fontSize: 15,
          )),
      child: Text("Cancelar"),
      onPressed: () => Navigator.of(context).pop(false),
    );

    Widget deleteButton = ElevatedButton(
      style: ElevatedButton.styleFrom(
        primary: Colors.red,
        textStyle: TextStyle(
          fontSize: 15,
        ),
      ),
      child: Text("Excluir"),
      onPressed: () async {
        await _deleteProducts();
        Navigator.pop(context, true);
      },
    );

    AlertDialog alertPopUp = AlertDialog(
      title: Text("Excluir os seguintes produtos?"),
      content: Container(
        height: 200,
        width: 300,
        child: Container(
          child: ListView.builder(
            //número de produtos selecionados
            //itemCount: this.products.length,
            itemCount: selectedProducts.length,
            itemBuilder: (context, index) {
              return ListTile(
                title: Text(selectedProducts[index].name),
                subtitle: Text(selectedProducts[index].description),
                trailing: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text('${selectedProducts[index].quantity} unid.'),
                      Text(convertNumToReais(selectedProducts[index].price)),
                    ]),
              );
            },
          ),
        ),
      ),
      scrollable: true,
      actions: [
        cancelButton,
        deleteButton,
      ],
    );

    return showDialog(
        context: context,
        builder: (context) {
          return alertPopUp;
        });
  }
}
