import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/models/PartnerModel.dart';
import 'package:apus_drones_mobile/models/ProductModel.dart';
import 'package:apus_drones_mobile/services/SearchService.dart';
import 'package:apus_drones_mobile/widgets/AddToCart.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:flutter/material.dart';

import '../Appbar.dart';
import '../../widgets/Searchbar.dart';
import '../CustomerProductDetailScreen.dart';
import 'ListOfProducts.dart';

class ListOfSearch extends StatefulWidget {
  final String text;
  const ListOfSearch(this.text, {Key? key}) : super(key: key);

  @override
  _ListOfSearchState createState() => _ListOfSearchState(text);
}

class _ListOfSearchState extends State<ListOfSearch> {
  String text;
  late List<ProductModel> products = [];
  late List<PartnerModel> partners = [];

  SearchService service = new SearchService();

  _fetch() {
    service.getSearch(text).then((r) => this.setState(() {
          products = r.products;
          partners = r.partners;
        }));
  }

  _ListOfSearchState(this.text) {
    _fetch();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: Appbar(),
      backgroundColor: Colors.white,
      body: Column(
        children: [
          Searchbar(),
          Expanded(
              child: SingleChildScrollView(
            child: showItens(),
          ))
        ],
      ),
      bottomNavigationBar: BotNavBar(role: UserType.CUSTOMER),
    );
  }

  showItens() {
    List<Widget> itens = [];
    List<Widget> partners = getPartnersWidgets();
    List<Widget> products = getProductsWidgets();
    partners.forEach((element) {
      itens.add(element);
    });
    products.forEach((element) {
      itens.add(element);
    });
    return Column(
      children: itens,
    );
  }

  Row getRowChildren(int i) {
    List<Widget> list = [];
    if (i >= partners.length) {
      return Row(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: list);
    }
    list.add(Card(
        elevation: 3,
        child: InkWell(
            onTap: () {
              PartnerModel p = partners[i];
              Navigator.push(context,
                  MaterialPageRoute(builder: (context) => ListOfProducts(p)));
            },
            child: Container(
                decoration: BoxDecoration(
                    border: Border.all(color: Colors.grey),
                    color: Colors.white),
                child: Column(
                  children: [
                    Padding(
                      padding: EdgeInsets.all(10),
                      child: Text(
                        partners[i].name,
                        style: TextStyle(
                            fontWeight: FontWeight.bold, fontSize: 16),
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.all(20),
                      child: Container(
                        width: 140.0,
                        height: 140.0,
                        decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            border: Border.all(width: 3.0, color: Colors.grey),
                            image: DecorationImage(
                                fit: BoxFit.fill,
                                image: NetworkImage(partners[i].avatarUrl))),
                      ),
                    )
                  ],
                )))));
    return new Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: list);
  }

  List<Widget> getPartnersWidgets() {
    List<Widget> list = [];

    list.add(Text(
      'Parceiros',
      style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
    ));
    for (var i = 0; i < partners.length; i += 2) {
      list.add(
        Padding(
          padding: EdgeInsets.fromLTRB(2, 30, 2, 0.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [getRowChildren(i), getRowChildren(i + 1)],
          ),
        ),
      );
    }

    return list;
  }

  List<Widget> getProductsWidgets() {
    List<Widget> list = [];
    list.add(Padding(padding: EdgeInsets.fromLTRB(0, 40, 0, 0)));
    list.add(Text(
      'Produtos',
      style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
    ));
    for (var i = 0; i < products.length; i += 2) {
      list.add(
        Padding(
          padding: EdgeInsets.fromLTRB(2, 30, 2, 0.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [getRowChildrenProduct(i), getRowChildrenProduct(i + 1)],
          ),
        ),
      );
    }

    return list;
  }

  Row getRowChildrenProduct(int i) {
    List<Widget> list = [];
    if (i >= products.length) {
      return Row(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: list);
    }

    list.add(Card(
        elevation: 3,
        child: InkWell(
            onTap: () => Navigator.push(
                  context,
                  MaterialPageRoute(
                      builder: (context) => CustomerProductDetailScreen(
                          productId: products[i].id)),
                ),
            child: Container(
                decoration: BoxDecoration(
                    border: Border.all(color: Colors.grey),
                    color: Colors.white),
                child: Column(
                  children: [
                    Padding(
                      padding: EdgeInsets.all(10),
                      child: Text(
                        products[i].name,
                        style: TextStyle(
                            fontWeight: FontWeight.bold, fontSize: 16),
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.all(10),
                      child: Text(
                        products[i].partner?.name ?? 'Desconhecido',
                        style: TextStyle(
                            fontWeight: FontWeight.bold, fontSize: 16),
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.all(10),
                      child: Text(
                        'R\$' + products[i].price.toString(),
                        style: TextStyle(
                            fontWeight: FontWeight.bold, fontSize: 16),
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.all(20),
                      child: Container(
                        width: 140.0,
                        height: 140.0,
                        decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            border: Border.all(width: 3.0, color: Colors.grey),
                            image: DecorationImage(
                                fit: BoxFit.fill,
                                image: NetworkImage(
                                    products[i].imageUrl ?? IMG_NO_PHOTO))),
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.all(10),
                      child: Text(
                        'Ver Detalhes',
                        style: TextStyle(
                            fontWeight: FontWeight.bold,
                            fontSize: 18,
                            color: Color.fromRGBO(0, 62, 161, 1)),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.all(10),
                      child: AddToCart(product: products[i]),
                    )
                  ],
                )))));
    return new Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: list);
  }
}
