import 'dart:convert';

import 'package:apus_drones_mobile/assets/config.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/AuthLoginModel.dart';
import 'package:apus_drones_mobile/providers/AuthProvider.dart';
import 'package:jwt_decoder/jwt_decoder.dart';
import 'package:http/http.dart' as http;

class AuthService {
  Future<Map<String, dynamic>> login(AuthLoginModel loginInfo) async {
    http.Response response = await http.post(
      Uri.parse('${Config.apiURL}/authenticate/login'),
      headers: {"Content-Type": "application/json"},
      body: json.encode(loginInfo.toJson()),
    );
    Map<String, dynamic> rsBody = json.decode(response.body);

    final token = rsBody["jwtToken"];
    if (token == null) {
      return {};
    }

    Map<String, dynamic> tokenData = JwtDecoder.decode(token);

    final provider = AuthProvider();
    await provider.setToken(token);

    UserType role = UserType.from[tokenData["Role"]];

    return {"accessToken": token, "role": role};
  }
}
