import 'package:apus_drones_mobile/assets/config.dart';
import 'package:apus_drones_mobile/models/CreateProductModel.dart';
import 'package:apus_drones_mobile/models/PartnerModel.dart';
import 'package:apus_drones_mobile/models/ProductModel.dart';
import 'package:apus_drones_mobile/models/UpdateProductModel.dart';
import 'package:apus_drones_mobile/utils/httpHeaders.dart';
import 'package:http/http.dart' as http;
import 'dart:async';
import 'dart:convert';

class ProductService {
  Future<ProductModel> getProduct(int id) async {
    http.Response response = await http.get(
      Uri.parse('${Config.apiURL}/product/$id'),
      headers: await getHttpHeaders(),
    );
    var rsBody = json.decode(response.body);
    ProductModel product = ProductModel.fromJson(rsBody);

    return product;
  }

  Future<List<ProductModel>> getProductsByPartner(PartnerModel partner) async {
    http.Response response = await http.get(
      Uri.parse('${Config.apiURL}/product?partner=${partner.id}'),
      headers: await getHttpHeaders(),
    );
    List<dynamic> rsBody = json.decode(response.body);
    List<ProductModel> products =
        rsBody.map((product) => ProductModel.fromJson(product)).toList();

    return products;
  }

  Future<void> postProduct(CreateProductModel product) async {
    try {
      http.Response response = await http.post(
          Uri.parse('${Config.apiURL}/product'),
          headers: await getHttpHeaders(),
          body: json.encode(product.toJson()));

      print('Response: ${response.statusCode}');
    } catch (error) {
      print('ERROR: $error');
    }
  }

  Future<void> patchProduct(UpdateProductModel product) async {
    try {
      final response = await http.patch(
          Uri.parse('${Config.apiURL}/product/${product.id}'),
          headers: await getHttpHeaders(),
          body: json.encode(product.toJson()));
    } catch (error) {
      print(error);
    }
  }

  Future<bool> deleteProduct(ProductModel product) async {
    try {
      http.Response response = await http.delete(
        Uri.parse('${Config.apiURL}/product/${product.id}'),
        headers: await getHttpHeaders(),
      );
      return response.statusCode == 200;
    } catch (error) {
      print(error);
      return false;
    }
  }
}
